@@ -32,9 +32,6 @@
 #include <stdio.h>
 #include <string.h>
 
-/* For getopt */
-#include <unistd.h>
-
 /* string.h must be included before gmp.h */
 #include "bignum.h"
 #include "buffer.h"
@@ -45,6 +42,8 @@
 #include "io.h"
 #include "rsa-session.h"
 
+#include "getopt.h"
+
 void
 rsa_session_set_encrypt_key(struct rsa_session *ctx,
 			    const struct rsa_session_info *key)
@@ -78,14 +77,17 @@ write_bignum(FILE *f, mpz_t x)
 {
   unsigned size = nettle_mpz_sizeinbase_256_u(x);
   uint8_t *p;
+  int res;
   
   if (!write_uint32(f, size))
     return 0;
   
-  p = alloca(size);
+  p = xalloc(size);
   nettle_mpz_get_str_256(size, p, x);
 
-  return write_string(f, size, p);
+  res = write_string(f, size, p);
+  free(p);
+  return res;
 }
 
 static int
@@ -95,8 +97,10 @@ process_file(struct rsa_session *ctx,
   uint8_t buffer[AES_BLOCK_SIZE * 100];
   unsigned leftover;
   unsigned padding;
-  
-  for (padding = leftover = 0; padding == 0;)
+
+  padding = leftover = 0;
+
+  for (;;)
     {
       size_t size = fread(buffer, 1, sizeof(buffer), in);
       if (ferror(in))
@@ -123,6 +127,14 @@ process_file(struct rsa_session *ctx,
 	  werror("Writing output failed: %s\n", strerror(errno));
 	  return 0;
 	}
+
+      if (padding)
+	{
+	  if (leftover)
+	    memcpy(buffer, buffer + size, leftover);
+
+	  break;
+	}
     }
   if (padding > 1)
     yarrow256_random(&ctx->yarrow, padding - 1, buffer + leftover);
@@ -130,6 +142,7 @@ process_file(struct rsa_session *ctx,
   buffer[AES_BLOCK_SIZE - 1] = padding;
   CBC_ENCRYPT(&ctx->aes, aes_encrypt, AES_BLOCK_SIZE, buffer, buffer);
   hmac_sha1_digest(&ctx->hmac, SHA1_DIGEST_SIZE, buffer + AES_BLOCK_SIZE);
+
   if (!write_string(out, AES_BLOCK_SIZE + SHA1_DIGEST_SIZE, buffer))
     {
       werror("Writing output failed: %s\n", strerror(errno));
